from seatsio.domain import ChartReport


class ChartReports:

    def __init__(self, http_client):
        self.http_client = http_client

    def by_label(self, chart_key):
        return self.__fetch_report("byLabel", chart_key)

    def by_category_key(self, chart_key):
        return self.__fetch_report("byCategoryKey", chart_key)

    def by_category_label(self, chart_key):
        return self.__fetch_report("byCategoryLabel", chart_key)

    def __fetch_report(self, report_type, chart_key):
        url = "/reports/charts/{key}/{reportType}"
        body = self.http_client.url(url, key=chart_key, reportType=report_type).get()
        return ChartReport(body)



from seatsio.domain import Workspace
from seatsio.pagination.listableObjectsClient import ListableObjectsClient
from seatsio.pagination.lister import Lister
from seatsio.pagination.pageFetcher import PageFetcher
from seatsio.workspaces.UpdateWorkspaceRequest import UpdateWorkspaceRequest
from seatsio.workspaces.createWorkspaceRequest import CreateWorkspaceRequest


class WorkspacesClient(ListableObjectsClient):

    def __init__(self, http_client):
        ListableObjectsClient.__init__(self, http_client, Workspace, "/workspaces")

    def create(self, name, is_test=None):
        response = self.http_client.url("/workspaces").post(
            CreateWorkspaceRequest(name, is_test))
        return Workspace(response.json())

    def update(self, key, name):
        self.http_client.url("/workspaces/{key}", key=key).post(
            UpdateWorkspaceRequest(name))

    def regenerate_secret_key(self, key):
        response = self.http_client.url("/workspaces/{key}/actions/regenerate-secret-key", key=key).post()
        return response.json()["secretKey"]

    def retrieve(self, key):
        return self.http_client.url("/workspaces/{key}", key=key).get_as(Workspace)

    def list(self, filter=None):
        page_fetcher = PageFetcher(Workspace, self.http_client, "/workspaces")

        if filter is not None:
            page_fetcher.set_query_param("filter", filter)

        return Lister(page_fetcher).list()



from six.moves.urllib.parse import quote, urlencode

import jsonpickle
import requests

from seatsio.exceptions import SeatsioException


class HttpClient:
    def __init__(self, base_url, secret_key, workspaceKey):
        self.baseUrl = base_url
        self.secretKey = secret_key
        self.workspaceKey = workspaceKey

    def url(self, relative_url, query_params=None, **path_params):
        if query_params is None:
            query_params = {}
        return ApiResource(self.secretKey, self.workspaceKey, self.baseUrl, relative_url, query_params, **path_params)


class ApiResource:
    def __init__(self, secret_key, workspaceKey, base_url, relative_url, query_params, **path_params):
        self.url = self.__create_full_url(base_url, relative_url, query_params, **path_params)
        self.secretKey = secret_key
        self.workspaceKey = workspaceKey

    def __create_full_url(self, base_url, relative_url, query_params, **path_params):
        for key in path_params:
            path_params[key] = quote(str(path_params[key]), safe='')
        full_url = base_url + relative_url.format(**path_params)
        if query_params:
            full_url += "?" + urlencode(query_params)
        return full_url

    def get(self):
        return GET(self.url, self.secretKey, self.workspaceKey).execute()

    def get_raw(self):
        return GET(self.url, self.secretKey, self.workspaceKey).execute_raw()

    def get_as(self, cls):
        return cls(self.get())

    def post(self, body=None):
        if body is None:
            return POST(self.url, self.secretKey, self.workspaceKey).execute()
        else:
            return POST(self.url, self.secretKey, self.workspaceKey).body(body).execute()

    def post_empty_and_return(self, cls):
        return cls(self.post().json())

    def delete(self):
        return DELETE(self.url, self.secretKey, self.workspaceKey).execute()


class GET:

    def __init__(self, url, secret_key, workspaceKey):
        self.httpMethod = "GET"
        self.url = url
        self.secret_key = secret_key
        self.workspaceKey = workspaceKey

    def execute(self):
        response = self.try_execute()
        if response.status_code >= 400:
            raise SeatsioException(self, response)
        else:
            return response.json()

    def execute_raw(self):
        response = self.try_execute()
        if response.status_code >= 400:
            raise SeatsioException(self, response)
        else:
            return response.content

    def try_execute(self):
        try:
            return requests.get(self.url, auth=(self.secret_key, ''), headers={'X-Workspace-Key': str(self.workspaceKey) if self.workspaceKey else None})
        except Exception as cause:
            raise SeatsioException(self, cause=cause)


class POST:

    def __init__(self, url, secret_key, workspaceKey):
        self.httpMethod = "POST"
        self.url = url
        self.secret_key = secret_key
        self.workspaceKey = workspaceKey
        self.bodyObject = None

    def body(self, body):
        self.bodyObject = body
        return self

    def execute(self):
        response = self.try_execute()
        if response.status_code >= 400:
            raise SeatsioException(self, response)
        else:
            return response

    def try_execute(self):
        try:
            json = jsonpickle.encode(self.bodyObject, unpicklable=False)
            return requests.post(
                url=self.url,
                auth=(self.secret_key, ''),
                headers={'X-Workspace-Key': str(self.workspaceKey) if self.workspaceKey else None},
                data=json
            )
        except Exception as cause:
            raise SeatsioException(self, cause=cause)


class DELETE:

    def __init__(self, url, secret_key, workspaceKey):
        self.httpMethod = "DELETE"
        self.url = url
        self.secret_key = secret_key
        self.workspaceKey = workspaceKey

    def execute(self):
        response = self.try_execute()
        if response.status_code >= 400:
            raise SeatsioException(self, response)

    def try_execute(self):
        try:
            return requests.delete(self.url, auth=(self.secret_key, ''), headers={'X-Workspace-Key': str(self.workspaceKey) if self.workspaceKey else None})
        except Exception as cause:
            raise SeatsioException(self, cause=cause)
